/*
The SciRoom code is available under the MIT license.
Made by Daniel Kumor

*****************************************************************
This file is the thing that makes the entire sciroom work without exploding.

All of the actual functionality is encapsulated in a "room" object, which is divided
into subcategories:

*/

//All of the functionality is encapsulated in the room class
var room = {
	
	//IMPORTANT GLOBALS
	chat:[],		//The single most important thing: The chat message array
	usrs:[],		//The users that were a part of the chat
	plugin:[],		//The plugin array
	rmid:'',		//The room to obtain data from
	
	//USER OPTIONS (Options that the user can set)
	animate:true,	//Whether to animate changes/posts
	enterpost:false,	//Whether the enter button posts, and shift-enter is new line, or vice versa
	
	
	addPlugin: function(plgin) {
		if (plgin.init)
			plgin.init();
		this.plugin.push(plgin);
	},
	
	//init:
	//	This function sets up the entire system, it includes all plugins, as well
	//	as all other things that are necessary for it to work
	init: function(){
		//We add jQuery, which is used extensively in all aspects of the script
		if (typeof jQuery == 'undefined') include('./code/jquery.js');
		
		//We include a beautiful plugin for tabs in textAreas by Ted Devito
		include('./code/tabby.js');
		//	This is absolutely necessary so that tabs don't explode when we press them
		
		//Includes the chat style
		include_style("./styles/chatstyle.css");
		
		//After the document finishes loading, we can set all bindings!
		window.onload = room.loaded;
	},
	//This function is called once the document is completely ready for manipulation
	loaded: function() {
		room.notify("Initializing...");
		
		//Obtains the room ID, if any
		room.rmid = gup('room');
		
		//Initializes all subsystems
		room.comm.init();		//Communications are first: settings are sent back
		
		//These systems do not require server-settings to initialize
		room.textinput.init();
		room.keyboard.init();
		room.window.init();
		room.timers.init();
		room.posts.init();
		
		room.notify(" ");
	},
	//This function is called once settings are obtained from server
	set: function() {
		//File requires server settings to initialize
		room.file.init();
	},
	
	
	/*
	Parser: This parses the input text and prepares it to display
	*/
	parser:{
		//The basic parser, does not do replacement of anything, just cleans the input
		//	if skipbr is true, then it does not replace newlines with <br>.
		basic:function(text,skipbr){
			//Gets rid of html tags
			text=text.replace(/</g,"&lt;").replace(/>/g,"&gt;");
			//if (!skipbr) text=text.replace(/\n/g,"<br>"); //added CSS pre
			return text;
		},
		//The main parser, sanitizes post input and uses plugins to change code
		main:function(text) {
			text=this.basic(text);
			var i=0;
			while (i<room.plugin.length) {
				if (room.plugin[i].parse)
					text=room.plugin[i].parse(text);
				i++;
			}
			return text;
		}
	},
	
	/*
	Comm: Communications manager, queries the server for new posts, sends posts on, etc
	*/
	comm:{
		islive:	false,			//Whether or not the page reached the bottom, and the user is in chat mode
		loadheight: 500,		//The distance from bottom at which to load new content
		isfastchat: true,		//This one is whether it is in fast-chat mode, or slowchat - they refresh at different rates
		fastchat: 1000,			//The time between queries in fastchat mode
		slowchat: 7000,			//The time between queries in slowchat mode
		lastpost: 0,			//The time at which the last post was received
		switchtime: 20000,		//The time to switch between chat-modes
		maxnum: 30,				//The maximum number of posts in a payload
		tobj: null,				//The timeout object
		postURL:'check',		//The URL of the data interface
		settingURL:'settings',	//The URL of settings
		fileURL:'file',			//The URL of the file uploader, if enabled
		init:function(){
			//Only initialize the database callbacks if there is a room to call back to!
			if (room.rmid) {
				//Callback to server for settings
				this.getsettings();
			}
			
			//Activate the 'post' button
			$("#postbutton").click(this.post);
		},
		getsettings:function() {
			$.get(room.comm.settingURL,{rid:room.rmid},room.comm.check_settings,'json').error(room.comm.failure);
		},
		check_settings:function(data) {
			if (data) {
				
				//Here the settings are set
				if (data.maxposts) room.comm.maxnum = data.maxposts;	//The amount of posts a full payload contains
				if (data.file_enabled) room.file.enabled = data.file_enabled;	//Whether or not the server allows file access
				if (data.file_ext) room.file.extensions = data.file_ext;		//The allowed extensions
				if (data.file_maxsize) room.file.maxsize = data.file_maxsize;		//The maximum file size the server allows
				
				room.set();			//Settings were loaded, run set
				room.comm.check();	//Obtain first batch of data
			} else {
				room.notify('Getting settings failed, retrying',2);
				window.setTimeout(room.comm.failtry,room.comm.getsettings);
			}
		},
		post:function(){
			var tobj = $('#inputtext');
			if ($('#usr').val()=='') $('#usr').val('Anonymous');
			if (tobj.val()!='') {
				val = room.comm.validate(tobj.val());
				if (!val) {
					if (room.rmid) {
						//This is serverside code - if a room is defined
						$.post(room.comm.postURL,{rid:room.rmid,usr:$("#usr").val(),txt:tobj.val(),date:(new Date()).getTime()}).success(function(){
							//Resets the file array
							room.file.clear();
							
							//Clears the textbox
							tobj.val('').focus();
							room.textinput.resize(true,true);
							//Since different chat speeds might make the post appear late, we prod the checker
							if (room.comm.tobj!=-1 && room.comm.tobj != 0) {
								clearTimeout(room.comm.tobj);	//Clears the current checking timer and immediately checks
								room.comm.check();
							}
						}).error(function() {room.notify('Failed to post!',2)});
					} else {
						//If there is no room defined, it means that it is in testing mode
						room.chat.push({text: tobj.val(),usr:$("#usr").val(),unum:1,pwd:'Offline Mode',date:(new Date()).getTime()});
						tobj.val('').focus();
						room.textinput.resize(true,true);
					}
					//room.window.scrolldown(true);
				} else {
					room.notify(val,2);
				}
			} else {
				room.notify("You didn't write anything!");
			}
		},
		check:function() {
			$.get(room.comm.postURL,{rid:room.rmid,pnum:room.chat.length},room.comm.check_response,'json').error(room.comm.failure);
		},
		check_response:function(data) {
			room.comm.tobj = -1;	//Currently ON
			if (data) {
				//The data is already taken from JSON,  so is good
				//We parse the data to obtain user numbers
				var i=0;
				var tmp=0;
				while (i< data.length) {
					tmp = room.usrs.indexOf(data[i].pwd);
					if (tmp==-1) {
						tmp = room.usrs.length;
						room.usrs.push(data[i].pwd);
					}
					//The +1 is there to make the first poster NOT white, since it looks weird
					data[i].unum=tmp+1;
					i++;
				}
				//Now we update the array
				room.chat=room.chat.concat(data);
				
				//The lastPost was NOW
				room.comm.lastpost = (new Date()).getTime();	//Sets the lastPost variable
			}
			
			//Sets the fastChat variable based on when the most recent post was given
			room.comm.isfastchat = (room.comm.lastpost + room.comm.switchtime > (new Date()).getTime());
			
			//Now to decide when the next update is to be
			if (!data || (data && data.length < room.comm.maxnum)) room.comm.islive=true;
				
			//After all parsing, set the next update timer
			if (room.comm.islive) {
				var time = room.comm.slowchat;
				if (room.comm.isfastchat) time = room.comm.fastchat;
				room.comm.tobj = window.setTimeout(room.comm.check,time);
			} else {
				room.comm.tobj = null;	//There is no update
			}
		},
		failtry: 5000,	//How much time between connection attempts
		failure: function(jqXHR, textStatus, errorThrown) {	//If there is a failure in communications
			room.notify("Connection Failure",2);
			window.setTimeout(room.comm.check,room.comm.failtry);
		},
		fupload: function(f) {
			if (room.file.enabled) {
				//We take the objects and upload them one by one to the server
				var xhr = new XMLHttpRequest();
				xhr.upload.addEventListener('progress', function(ev) {
					//TODO: File Upload
					//$('#fileUpload tr');
				});
			}
		},
		validate:function(text){
			var i=0;
			while (i<room.plugin.length) {
				if (room.plugin[i].validate) {
					var result=room.plugin[i].validate(text);
					if (result != true && typeof result !='undefined')
						return result;
				}
				i++;
			}
			return 0;
		}
	},
	
		
	/*
	Post manager: Displays posts and does all modifications to the posts
	*/
	posts:{
		updatetime:100,		//The time to wait between update checks
		num:0,				//The number of posts currently displayed
		colornum:4,			//The number of colors available
		newnum:0,			//The number of new posts since last clear
		
		updatelist:[],		//The list of changes to make
		
		//The post code:
		//Places where content is to be added is denoted by %. The codes are:
		//%c color number
		//%u username
		//%p user password
		//%d date
		//%t inner content
		//%b box content, so postcode that goes inside box
		boxcode:'<div class="chat p%c_color" style="display:none"><div class="chat_pwd">%p</div><div class="chat_name">%u</div>%b</div>',
		postcode:'<div class="chat_date">%d</div><div class="ccontainer"><div class="chat_content" style="display:none">%t</div></div>',
		
		boxatime:120,	//Animation times for posts and boxes
		postatime:70,
		
		//Initializes it..
		init:function() {
			room.posts.update();
		},
		update:function(){
			var scroll=false;	//Whether to scroll at the end
			var i=0;			//An iterator
			
			//Display new posts
			if (room.posts.num<room.chat.length) {	//If there are any posts that are not displayed
				scroll=true;			//We want to check for scroll, since we we will add a post
				while (room.posts.num<room.chat.length) {
					room.posts.addpost(room.posts.num);	//Adds the post, and then increments
					room.posts.runplugins(room.posts.num);
					room.posts.num++;
					room.posts.newnum++;
				}
			}
			//Update existing posts
			while (i<room.posts.updatelist.length) {
				//The first element is the post number to make change to.
				//	Second element is a chat object
				var co = room.posts.updatelist[i][1];
				var nu = room.posts.updatelist[i][0];
				if (co.text) {
					scroll=true;	//The new text might have a different size
					$(".chat_content:eq("+nu+")").html(room.parser.main(co.text));
					room.posts.runplugins(nu);
				}
				if (co.date) {
					$(".chat_date:eq("+nu+")").text(co.date);
				}
				i++;
			}
			room.posts.updatelist=[];	//Clears the list
			
			if (scroll) {	//Scroll means that at least one post was added
				room.window.scrolldown();	//Scrolls down if post added and if at the bottom.
				//Scrolldown changes newnum if the user is at the bottom
				room.posts.pulsetitle();
			}
			
			window.setTimeout(room.posts.update,room.posts.updatetime);	//Sets timeout for next update
		},
		pulsetitle: function() {
			if (room.comm.islive == false) room.posts.newnum=0;	//If the room is not live, then don't bother
			if (room.posts.newnum !=0) {
				if (!room.posts.pulseenabled)
					room.posts.pulser();
				$('#roomname').text(room.rmid + ' (' + room.posts.newnum+' new)');
			} else {
				$('#roomname').text(room.rmid);
			}
		},
		pulseenabled: false,	//Internal variable: whether pulse is enabled
		pulseloc: true,		//The location in the pulse (tic toc)
		pulsetime: 5500,		//The time to pulse through
		pulser: function() {
			if (room.posts.newnum !=0 && room.window.active == false) {
				room.posts.pulseenabled=true;
				if (room.posts.pulseloc) {
					if (room.posts.newnum==1) {room.window.writetitle('new post! - '+room.rmid);}
					else {room.window.writetitle(''+ room.posts.newnum + ' new posts! - ' + room.rmid);}
				} else {room.window.writetitle();}
				room.posts.pulseloc=!room.posts.pulseloc;
				window.setTimeout(room.posts.pulser,room.posts.pulsetime);
			} else {
				room.posts.pulseenabled=false;
				room.window.writetitle();
			}
		},
		parsecode:function(code,c) {	//Code text and object to replace it with
			return code.replace(/%u/g,room.parser.basic(c.usr)).replace(/%p/g,room.parser.basic(c.pwd)).replace(/%t/g,room.parser.main(c.text)).replace(/%d/g,prettytime(c.date)).replace(/%c/g,c.unum%room.posts.colornum);
		},
		addpost:function(cnum) {
			var c=room.chat[cnum];
			posttext=room.posts.parsecode(room.posts.postcode,c);
			if (cnum==0 || room.chat[cnum-1].unum!=c.unum){	//If we make a new post box
				posttext=room.posts.parsecode(room.posts.boxcode.replace(/%b/g,posttext),c);
				$("#chatmsgs").append(posttext);
				$(".chat_content:last").show();
				if (room.animate)
					$(".chat:last").slideToggle(room.posts.boxatime);
				else
					$(".chat:last").show();
			} else {	//If we just add to the last postbox
				$("#chatmsgs .chat:last").append(posttext);
				if (room.animate)
					$(".chat_content:last").slideToggle(room.posts.postatime);
				else
					$(".chat_content:last").show();
			}
			
		},
		runplugins:function(cnum){
			var i=0;
			var cntnt = $(".chat_content:eq("+cnum+")")
			while (i<room.plugin.length) {
				if (room.plugin[i].run)
					text=room.plugin[i].run(cntnt);
				i++;
			}
		}
	},
	
	//textinput:
	//	All of the methods associated with the text input
	textinput : {		
		addsize:34,		//The amount to add to the current size on resize
		
		a_time:200,		//Time for resize animation
		
		//The initializer function
		init:function(){	
			var txt =$("#inputtext");		
			//This is the most unusual workaround for finding the size necessary for the textarea
			//	Since the div will automatically expand to include text, we can get the height the
			//	textarea needs to be in order to fit all the text. Also, this will easily double as a
			//	"preview post" area.
			$("#chatInput").append("<div id='preview_area' style='width:99%; white-space: pre; display:none; font:"+txt.css('font-size')+' '+txt.css('font-family').replace(/'/g,'')+";'></div>");
		},
		
		//This function resizes the textbox to fit all text. If force is true, then it makes the
		//	textbox smaller if text not large enough to fill it all
		resize:function(force,noscroll){
			var pobj = $('#preview_area');
			var tobj = $('#inputtext');
			//Sets the preview object's content to be simply parsed input
			pobj.html(room.parser.basic(tobj.val()));
			
			//get the height
			var h= parseInt(pobj.css("height").replace('px',''));
			
			//Now, we don't lower the textbox height unless we actually have to
			if (force==1||parseInt(tobj.css("height").replace('px',''))<h+room.textinput.addsize){
				var newh = h+room.textinput.addsize+'px';
				if (room.animate)
					tobj.animate({height: newh},room.textinput.a_time);
				else
					tobj.css("height",newh);
			}
			if (!noscroll)
				room.window.scrolldown();
		},
		insert: function(text) {
			var txt = $('#inputtext').get(0);
			var sp = txt.selectionStart;
			
			txt.value = txt.value.substring(0,sp)
						+ text
						+ txt.value.substring(sp,txt.value.length);
			txt.selectionStart = sp;txt.selectionEnd=sp;
		},
		inserttab:function() {
			//We will now use the pretty plugin by Ted Devito!
			tabby.tab($('#inputtext')[0],room.keyboard.shift_c);
		}
	},

	/*
	Keyboard:Handles all interactions with the keyboard
	*/
	keyboard : {
		shift_c:false,		//The current and previous key values
			shift_p:false,
		enter_c:false,
			enter_p:false,
		keynum:0,			//The total number of keys pressed
		txtresize: 90,		//Each X keys pressed resize the textbox
		
		updn:-1,		//Whether or not to allow up/down keys to obtain previous posts
		
		mouseX:0,		//The X and Y location of the mouse
		mouseY:0,
		
		init:function(){
			$('#inputtext').keydown(room.keyboard.keydn).keyup(room.keyboard.keyup).bind('paste',function(e){window.setTimeout(room.textinput.resize,10)});
			document.onmousemove = room.keyboard.mouse;
		},
		mouse: function(e) {
			room.keyboard.mouseX = e.clientX;
			room.keyboard.mouseY = e.clientY;
			//$('#usr').val('' + room.keyboard.mouseX + ','+room.keyboard.mouseY);
		},
		keyup: function(event) {
			var t = room.keyboard;
			var change=false;
			switch(event.keyCode) {
				case 13:	//Enter
					t.enter_p=t.enter_c;
					t.enter_c=false;
					if (t.enter_p!=t.enter_c) change=true;
					break;
				case 16:	//Shift
					t.shift_p=t.shift_c;
					t.shift_c=false;
					if (t.shift_p!=t.shift_c) change=true;
					break;
			}
		},
		keydn: function(event) {
			var t = room.keyboard;
			var change=false;
			switch(event.keyCode) {
				case 13:	//Enter
					t.enter_p=t.enter_c;
					t.enter_c=true;
					if (t.enter_p!=t.enter_c) change=true;
					break;
				case 16:	//Shift
					t.shift_p=t.shift_c;
					t.shift_c=true;
					if (t.shift_p!=t.shift_c) change=true;
					break;
				case 9:		//Tab
					event.preventDefault();
					room.textinput.inserttab(t.shift_c);	//Insert tab, and if shift, then untab
					break;
				case 38:	//upkey							//Up and Down keys are here for allowing user to obtain text of previous post
					var tobj = $('#inputtext');
					//If we can enable updown movement
					if (tobj.val()=='') {room.keyboard.updn=room.chat.length;}
					//If enabled, and there is a post before this, do it!
					if (room.keyboard.updn!=-1 && room.keyboard.updn !=0) {
						event.preventDefault();
						tobj.val(room.chat[--room.keyboard.updn].text);
					}
					break;
				case 40:	//downkey
					if (room.keyboard.updn!=-1 && room.keyboard.updn !=room.chat.length-1) {
						event.preventDefault();
						$('#inputtext').val(room.chat[++room.keyboard.updn].text);
					} else if (room.keyboard.updn ==room.chat.length-1) {
						//The person moved back to the original post, so let's give a clean
						//	post
						event.preventDefault();
						room.keyboard.updn=-1;
						$('#inputtext').val('');
					}
					break;
				default:
					room.keyboard.updn=-1;
					break;
			}
			if (change) t.keycheck(event);
			
			//Resize text input if this amount of keys pressed
			if (++t.keynum%t.txtresize==0)
				room.textinput.resize();
		},
		
		keycheck:function(evt) {
			if (this.shift_c && this.enter_c)
				if (room.enterpost) this.newline(evt);
				else this.post(evt);
			else if (this.enter_c)
				if (room.enterpost) this.post(evt);
				else this.newline(evt);
		},
		
		newline:function(evt) {
			room.textinput.resize();
		},
		
		post:function(evt) {
			room.comm.post();
			evt.preventDefault();
		}
		
	},
	
	/*
	Timers: Basic timed functions happen here
	*/
	timers:{
		init:function(){
			this.checktimes();
		},
		checktimes_t:20000,
		checktimes:function() {
			//The function that checks times
			var i=0;
			while (i<room.chat.length) {
				room.posts.updatelist.push([i,{date: prettytime(room.chat[i].date)}]);
				i++;
			}
			window.setTimeout(room.timers.checktimes,room.timers.checktimes_t);
		}
	},
	
	/*
	Notification: the room's status message
	*/
	notification:{text:"",wait:10000,
		reset:function(){$("#postinfo").text(room.notification.text);}
	},
	//The function which puts text in the notification area
	notify:function(text,type){
		if (!text) text=" ";	//Just because it failed when given null
		switch (type) {
			case 1:
				this.notification.text=text;
				this.notification.reset();
				break;
			case 2:
				$("#postinfo").html("<span style='color:red'>"+text+"</span>");
				window.setTimeout(this.notification.reset,this.notification.wait);
				break;
			default:
				$("#postinfo").html(text);
				window.setTimeout(this.notification.reset,this.notification.wait);
				break;	
		}
	},
	/*
		Functions that enable file access
	*/
	file:{
		enabled:false,		//Whether or not the server allows the upload of files
		maxsize:0,			//The maximum file size the server allows
		extensions:[],		//List of extensions to allow if enabled. If nothing there, allow all extensions
		
		files: [],			//The file objects
		
		//Workaround for enter/leave so that leave is real leave
		enters: 0,			//The amount of times enter event
		leaves: 0,			//The amount of leave events
		init: function() {
			//If the browser does not have File API, then file drag drop is poop
			if (typeof window.FileReader === 'undefined') this.enabled = false;
			
			if (this.enabled) {
				/*
					The first thing that is changed is the document's response to dragging,
					it disables drop on the body, as not to asplode if someone accidentally drops in the wrong place
				*/
				bdy = document;
				bdy.ondragenter = function() {
					//Start the drag only the first time
					if (room.file.enters==room.file.leaves) {
						if (navigator.userAgent.toLowerCase().indexOf('firefox')!=-1) {
							room.notify('ffx',2);	//Firefox, y u work in unpredictable ways?
							room.file.leaves=-1;	//	Since firefox did not work well with enter/leave, I had to create an
							//	alternate way to display drop files than in chrome. Shame, since the chrome way is cooler
							room.notify('Drop File in Input Box');
						} else {
							room.file.startDrag();
							$('#chatInput').addClass('drag');
						}
					}
					room.file.enters++;
					return false;
				};
				bdy.ondragleave = function() {
					//A workaround for leaving the window: it 'leaves' when entering the drop area, which was impossible to
					//	detect well, so it just fires a timer, and if the event is not canceled by 'enter' within the time,
					//	then it leaves!
					room.file.leaves++;
					if (room.file.leaves==0) room.file.leaves = -1;
					if (room.file.enters==room.file.leaves) {
						room.file.endDrag();
					}
					return false;
				};
				//Disables drop for the document, so that the browser does not fail us
				bdy.ondragover = function() {return false;};
				bdy.ondrop = function(e) {e.preventDefault();};
				
				//Initializes the file drag+drop stuffs
				dragger = document.getElementById('chatInput');
				//This needs to be enabled for dragdrop to work
				dragger.ondragover = function() {return false;};
				dragger.ondragenter = function() {
					room.file.enters++;
					if (room.file.leaves==-1) {
						room.file.startDrag();
					}
					$('#chatInput').addClass('dragIn').removeClass('drag');
					return false;};
				dragger.ondragleave = function() {
					if (room.file.leaves==-1) {
						room.file.endDrag();
					} else {
						room.file.leaves++;
						$('#chatInput').addClass('drag').removeClass('dragIn');
					}
					return false;
				};
				dragger.ondragend = function() {
					return false;
				};
				dragger.ondrop = room.file.drop;
			}
		},
		drop: function(e) {
			e.preventDefault();	//Don't want any failbusiness
			room.file.endDrag();
			
			var links = e.dataTransfer.getData('text/uri-list');
			if (links)
				room.textinput.insert(links);
			
			//Add on to the file array
			var files = e.dataTransfer.files
			for (var i=0, il=files.length; i<il; i++) {
				if (room.file.isallowed(files[i].name)) {
					if (room.file.maxsize==0 || files[i].size <= room.file.maxsize) {
						room.file.files.push(files[i]);
					} else {
						room.notify('File: ' + files[i].name + ' is too large!',2);
					}
				} else {
					room.notify('File: ' + files[i].name + ' is not allowed.',2);
				}
			}
			
			//If there are any files to upload, show them
			if (room.file.files.length!=0)
				room.file.showpanel()
			
			return false;
		},
		showpanel: function() {
			$('#fileUpload').html('').width(180);
			$('#inputtext').width($('#body').width()-200);
			if (room.file.files.length==0) {
				roo.file.clear();
			} else if (room.file.files.length==1 && room.file.isimg(room.file.files[0].type)) {
				reader = new FileReader();
				reader.onload = function(e) {
					//Show the pic
					$('#fileUpload').html('<img onload="setTimeout(room.file.heq,100)" align="center" src="'+ e.target.result +'" width="170">');
				}
				reader.readAsDataURL(room.file.files[0]);
			} else {
				//Show the file list
				var list = '<table border="0" align="center">'
				for (var i=0, il = room.file.files.length;i<il;i++) {
					list = list + '<tr><td>'+room.file.files[i].name 
						+ "</td><td><a href='javascript:void(0)' onclick='room.file.remove("+i+")'>x</a></td></tr>";
				}
				$('#fileUpload').html(list+'</table>');
				
			}
			
			//Now equalize the height:
			room.file.heq();
		},
		remove: function(rm) {
			room.file.files.splice(rm,1);	//Remove the file
			room.file.showpanel();			//Remake the panel
		},
		heq: function() {
			var h = $('#fileUpload').height();
			if (h>$('#inputtext').height()) {
				$('#inputtext').height(h);
			}
		},
		clear: function() {
			$('#fileUpload').html('').width(0);
			$('#inputtext').width($('#body').width()-5);
			room.file.files=[];
		},
		startDrag: function() {
			$('#inputtext').addClass('invisible');
			$('#fileUpload').addClass('invisible');
		},
		endDrag: function() {
			room.file.enters=0;
			if (room.file.leaves>0)
				room.file.leaves=0;
			$('#chatInput').removeClass('drag').removeClass('dragIn');
			$('#inputtext').removeClass('invisible');
			$('#fileUpload').removeClass('invisible');
			
		},
		isallowed: function(fname) {
			if (room.file.extensions.length==0) return true;
			i=0;
			while (i<room.file.extensions.length) {
				if (room.file.extmatch(fname,room.file.extensions[i])) return true;
				i++;
			}
			return false;
		},
		isimg: function(f) {	//Checks if a file is an image
			if (f.indexOf('image')!=-1) return true;
			return false;
		},
		extmatch: function(str, suffix) {
    		return (str.toLowerCase().indexOf(suffix, str.length - suffix.length) != -1);
    	}
	},
	/*
		Functions that change the window
	*/
	window:{
		active: true,	//Whether the browser window is active or inactive
		
		init:function(){
			$(window).scroll(this.scrollbind);	//Binds the scroll event
			
			//Sets the room title
			this.writetitle();
			$('#roomname').text(room.rmid);
			
			//Set the active/Not active value functions
			$(window).focus(function() {
				room.window.active = true;
				room.window.writetitle();	//Clear the title to be the original if the window is now active
			});
			$(window).blur(function() {
				room.window.active = false;
			});
			
		},
		//Writes the window's title to be the necessary thing
		writetitle: function(title) {
			if (title) {
				document.title = title
			} else {
				document.title = room.rmid + " - SciRoom";
			}
		},
		scrolldisabled:false,	//Boolean, whether it should automatically scroll
		scrollatime:100,			//The scrolling animation time
		scrolldown:function(undis){	//Undis forces scrolldown
			if ((!this.scrolldisabled || undis) && room.window.active) {
				room.posts.newnum=0;	//There are no new posts, since we are at the bottom
				//var len=document.body.scrollHeight;
				var len=$("#pg")[0].scrollHeight;	//For some reason the provious didnt work in FF
				if (room.animate) {
					$('html, body').animate({scrollTop: len},this.scrollatime);
				} else
					$('html, body').scrollTop(len);
			}
		},
		lastload:0,	//The time of the last 'load'
		waittime:40,//The time to wait
		scrollbind:function(){	//This function is binded to the scroll event
			var enabled = (($(window).scrollTop() + $(window).height() >= $(document).height()))
			room.window.scrolldisabled = !enabled;
			if (enabled) {
				//The user scrolled down all the way to the bottom, so there are no new posts
				room.posts.newnum=0;
				room.posts.pulsetitle();
			} 
			if (!room.comm.islive) {
				//if we don't have live communication, we check how far from the bottom the user is, and
				//	if the user is within a certain distance from the bottom, we load new content
				var height = $(document).height() - ($(window).scrollTop() + $(window).height());
				//This is the height from the bottom
				var time = (new Date()).getTime();
				//This is the time
				if (height < room.comm.loadheight && room.window.lastload + room.window.waittime < time) {
					room.window.lastload = time;
					room.comm.check();
				}
				
			}
		}
	}
}

/*
These allow the inclusion of script/style files without putting them in the header directly.
*/
function include(scriptsrc) {
	var html_doc = document.getElementsByTagName('head').item(0)	;
    var js = document.createElement('script')						;
	js.setAttribute('language', 'javascript')						;
	js.setAttribute('type', 'text/javascript')						;
	js.setAttribute('src', scriptsrc)								;
	html_doc.appendChild(js)										;
}
function include_style(stylesrc) {
	var html_doc = document.getElementsByTagName('head').item(0)	;
    var js = document.createElement('link')							;
	js.setAttribute('rel', 'stylesheet')							;
	js.setAttribute('type', 'text/css')								;
	js.setAttribute('href', stylesrc)								;
	html_doc.appendChild(js)										;
}

//This function allows parsing of URL parameters -
//http://www.netlobo.com/url_query_string_javascript.html
function gup( nam ) {
  nam = nam.replace(/[\[]/,"\\\[").replace(/[\]]/,"\\\]");
  var regexS = "[\\?&]"+nam+"=([^&#]*)";
  var regex = new RegExp( regexS );
  var results = regex.exec( window.location.href );
  if( results == null )
    return "";
  else
    return results[1];
}

/*
Extra functions: Functions that are necessary, but don't really fit anywhere else
*/
//Makes the dates and times look pretty
function prettytime(time) {
	var diff = (new Date()).getTime() - time;
	if (diff < 0) return "from the future";
	if (diff < 1000) return "just now";
	diff=Math.round(diff/1000);
	if (diff < 2) return "a second ago";
	if (diff>10)
		diff=10*Math.round(diff/10);
	if (diff < 60) return diff +" seconds ago";
	diff = Math.round(diff/60);
	if (diff < 2) return "a minute ago";
	if (diff < 60) return diff + " minutes ago";
	diff = Math.round(diff/60);
	if (diff <2) return "an hour ago";
	if (diff <24) return diff + " hours ago";
	diff=Math.round(diff/24);
	if (diff <2) return "Yesterday";
	if (diff <7) return diff + " days ago";
	diff=Math.round(diff/7);
	if (diff < 2) return "a week ago";
	return diff + " weeks ago";
}

